@@ -2,21 +2,21 @@
 
 # Copyright (c) 2009, Purdue University
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
-# 
+#
 # Redistributions of source code must retain the above copyright notice, this
 # list of conditions and the following disclaimer.
 #
 # Redistributions in binary form must reproduce the above copyright notice, this
 # list of conditions and the following disclaimer in the documentation and/or
 # other materials provided with the distribution.
-# 
+#
 # Neither the name of the Purdue University nor the names of its contributors
 # may be used to endorse or promote products derived from this software without
 # specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -35,7 +35,7 @@ Allows client to connect and run arbitrary functions in core.py.
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.12'
+__version__ = '0.14'
 
 
 import datetime
@@ -43,6 +43,9 @@ import os
 import inspect
 import SocketServer
 import time
+import traceback
+import uuid
+import xmlrpclib
 
 import roster_core
 
@@ -67,7 +70,8 @@ class Server(object):
   """Daemon library used to serve commands to the client."""
   def __init__(self, config_instance, keyfile=None, certfile=None,
                inf_renew_time=None, core_die_time=None,
-               clean_time=None, unittest_timestamp=None):
+               clean_time=None, unittest_timestamp=None,
+               unit_test=False):
     """Sets up config instance. Stores core instances.
 
     Inputs:
@@ -77,6 +81,7 @@ class Server(object):
        inf_renew_time: time to refresh infinite credentials (seconds)
        core_die_time: time for each core instance to die (seconds)
        clean_time: time to wait between core instance cleanings (seconds)
+       unit_test: boolean indicating a unit-test is being run.
     """
     self.config_instance = config_instance
     self.keyfile = keyfile
@@ -96,6 +101,8 @@ class Server(object):
     if( core_die_time is None ):
       self.core_die_time = self.config_instance.config_file['server'][
           'core_die_time']
+    self.log_file = self.config_instance.config_file['server'][
+        'server_log_file']
     self.get_credentials_wait_increment = self.config_instance.config_file[
         'server']['get_credentials_wait_increment']
     self.server_killswitch = self.config_instance.config_file['server'][
@@ -104,12 +111,46 @@ class Server(object):
     if( clean_time is None ):
       self.clean_time = self.core_die_time
     self.cred_cache_instance = credentials.CredCache(self.config_instance,
-                                   inf_renew_time)
+                                   inf_renew_time, unit_test)
     self.unittest_timestamp = unittest_timestamp
     self.core_store = [] # {'user': user, 'last_used': last_used, 'instance': }
     self.get_credentials_wait = {} # {'user1': 3, 'user2': 4}
     self.last_cleaned = datetime.datetime.now()
 
+  def LogException(self, function, args, kwargs, user_name):
+    """Save functions traceback to logfile
+
+    Inputs:
+      function: string of function name
+      args: args list
+      kwargs: keyword args dict
+      user_name: username string
+
+    Outputs:
+      str: uuid string from logfile
+    """
+    uuid_string = str(uuid.uuid4())
+    if( not os.path.exists ):
+      try:
+        open(self.log_file, 'w').close()
+      except:
+        raise ServerError('Could not write to logfile "%s"' % self.log_file)
+    log_file_handle = open(self.log_file, 'a')
+    log_file_handle.writelines('\n\n---------------------\n')
+    log_file_handle.writelines(uuid_string)
+    log_file_handle.writelines('\n')
+    log_file_handle.writelines('FUNCTION: %s\n' % function)
+    log_file_handle.writelines('ARGS: %s\n' % args)
+    log_file_handle.writelines('KWARGS: %s\n' % kwargs)
+    log_file_handle.writelines('USER: %s\n' % user_name)
+    log_file_handle.writelines('TIMESTAMP: %s\n\n' % (
+        datetime.datetime.now().isoformat()))
+    traceback.print_exc(None, log_file_handle)
+    log_file_handle.writelines('\n---------------------\n')
+    log_file_handle.close()
+
+    return uuid_string
+
   def CleanupCoreStore(self):
     """Cleans up expired instances in core_store"""
     delete_list = []
@@ -143,6 +184,9 @@ class Server(object):
       for index, item in enumerate(object_to_convert):
         object_to_convert[index] = self.StringToUnicode(item)
       converted_object = object_to_convert
+    elif( isinstance(object_to_convert, xmlrpclib.DateTime) ):
+      converted_object = datetime.datetime.strptime(
+          object_to_convert.value, "%Y%m%dT%H:%M:%S")
     return converted_object
 
   def GetCoreInstance(self, user_name):
@@ -184,6 +228,7 @@ class Server(object):
                              'new_credential': u'
                                  be4d4ecf-d670-44a0-b957-770e118e2755'}
     """
+    uuid_string = None
     credfile = unicode(credfile)
     user_name = unicode(user_name)
     ## Instantiate the core instance
@@ -212,22 +257,28 @@ class Server(object):
         raise FunctionError('Function does not exist.')
 
       types = inspect.getargspec(run_func)
-      ## Figure out what each function is expecting
-      if( types[3] is None and len(types[0]) == 1 ):
-        # Nothing in core function
-        core_return = run_func()
-      elif( types[3] is None and len(types[0]) != 1 ):
-        # Arguments only in core function
-        core_return = run_func(*args)
-      elif( types[3] is not None and (len(types[0]) - len(types[3])) == 1 ):
-        # KWArguments only in core function
-        core_return = run_func(**kwargs)
-      elif( types[3] is not None and (len(types[0]) - len(types[3])) != 1 ):
-        # Both kwargs and args in core function
-        core_return = run_func(*args, **kwargs)
-      else:
-        raise ArgumentError('Arguments do not match.')
-      core_return = {'core_return': core_return, 'new_credential': cred_status}
+      try:
+        ## Figure out what each function is expecting
+        if( types[3] is None and len(types[0]) == 1 ):
+          # Nothing in core function
+          core_return = run_func()
+        elif( types[3] is None and len(types[0]) != 1 ):
+          # Arguments only in core function
+          core_return = run_func(*args)
+        elif( types[3] is not None and (len(types[0]) - len(types[3])) == 1 ):
+          # KWArguments only in core function
+          core_return = run_func(**kwargs)
+        elif( types[3] is not None and (len(types[0]) - len(types[3])) != 1 ):
+          # Both kwargs and args in core function
+          core_return = run_func(*args, **kwargs)
+        else:
+          raise ArgumentError('Arguments do not match.')
+      except Exception, e:
+        uuid_string = self.LogException(function, args, kwargs, user_name)
+        return {'log_uuid_string': uuid_string, 'error': str(e),
+                'core_return': None, 'new_credential': None}
+      core_return = {'core_return': core_return, 'new_credential': cred_status,
+                     'log_uuid_string': uuid_string, 'error': None}
     else:
       core_return = 'ERROR: Invalid Credentials'
     return core_return
@@ -243,18 +294,24 @@ class Server(object):
       string: string of credential
               example: u'be4d4ecf-d670-44a0-b957-770e118e2755'
     """
-    user_name = unicode(user_name)
-    core_instance = self.GetCoreInstance(user_name)
-    cred_string = self.cred_cache_instance.GetCredentials(user_name, password,
-                                                          core_instance)
-    if( cred_string == '' ):
-      if( not self.get_credentials_wait.has_key(user_name) ):
-        self.get_credentials_wait.update({user_name: 0})
-      time.sleep(self.get_credentials_wait[user_name])
-      self.get_credentials_wait[user_name] = self.get_credentials_wait[
-          user_name] + self.get_credentials_wait_increment
-    elif( self.get_credentials_wait.has_key(user_name) ):
-      self.get_credentials_wait.pop(user_name)
+    try:
+      user_name = unicode(user_name)
+      core_instance = self.GetCoreInstance(user_name)
+      cred_string = self.cred_cache_instance.GetCredentials(user_name, password,
+                                                            core_instance)
+      if( cred_string == '' ):
+        if( not self.get_credentials_wait.has_key(user_name) ):
+          self.get_credentials_wait.update({user_name: 0})
+        time.sleep(self.get_credentials_wait[user_name])
+        self.get_credentials_wait[user_name] = self.get_credentials_wait[
+            user_name] + self.get_credentials_wait_increment
+      elif( self.get_credentials_wait.has_key(user_name) ):
+        self.get_credentials_wait.pop(user_name)
+    except Exception, e:
+      uuid_string = self.LogException(
+          'GetCredentials', [user_name, '<password>'], {}, user_name)
+      return {'log_uuid_string': uuid_string, 'error': str(e),
+              'core_return': None, 'new_credential': None}
 
     return cred_string
 
@@ -267,11 +324,17 @@ class Server(object):
     Outputs:
       bool: bool of valid string
     """
-    user_name = unicode(user_name)
-    credstring = unicode(credstring)
-    core_instance = self.GetCoreInstance(user_name)
-    valid = self.cred_cache_instance.CheckCredential(
-        credstring, user_name, core_instance)
+    try:
+      user_name = unicode(user_name)
+      credstring = unicode(credstring)
+      core_instance = self.GetCoreInstance(user_name)
+      valid = self.cred_cache_instance.CheckCredential(
+          credstring, user_name, core_instance)
+    except Exception, e:
+      uuid_string = self.LogException(
+          'IsAuthenticated', [user_name, '<credstring>'], {}, user_name)
+      return {'log_uuid_string': uuid_string, 'error': str(e),
+              'core_return': None, 'new_credential': None}
     if( valid == '' ):
       return True
     return False
